@@ -3,13 +3,14 @@
 extern crate rustc_interface;
 
 use clap::clap_app;
+use rand::prelude::*;
 use std::{
   env,
   path::PathBuf,
   process::{exit, Command},
 };
 
-const VERSION: &'static str = env!("CARGO_PKG_VERSION");
+const VERSION: &str = env!("CARGO_PKG_VERSION");
 
 fn main() {
   let flowistry_rustc_path = std::env::current_exe()
@@ -74,33 +75,55 @@ fn main() {
   };
   args.push(("COMMAND", cmd));
 
-  let mut file_path = PathBuf::from(file_name);
-  let mut package = None;
-  while let Some(parent) = file_path.parent() {
-    file_path = parent.to_path_buf();
+  let file_path = PathBuf::from(file_name);
+  let metadata = cargo_metadata::MetadataCommand::new()
+    .no_deps()
+    .other_options(["--offline".to_string()])
+    .exec()
+    .unwrap();
+  let (pkg, target) = metadata
+    .workspace_members
+    .iter()
+    .filter_map(|pkg_id| {
+      let pkg = metadata
+        .packages
+        .iter()
+        .find(|pkg| &pkg.id == pkg_id)
+        .unwrap();
 
-    let manifest_path = file_path.join("Cargo.toml");
-    if manifest_path.exists() {
-      let manifest = cargo_toml::Manifest::from_path(manifest_path).unwrap();
-      package = Some(manifest.package.unwrap().name);
-      break;
-    }
-  }
+      let target = pkg
+        .targets
+        .iter()
+        .filter(|target| file_path.starts_with(target.src_path.parent().unwrap()))
+        .max_by_key(|target| target.src_path.components().count());
+
+      target.map(move |target| (pkg, target))
+    })
+    .next()
+    .unwrap_or_else(|| panic!("Could not find target for path: {}", file_name));
 
   let mut cmd = Command::new(cargo_path);
   cmd
-    .arg("check")
-    .arg("-q")
-    .args(flags)
-    .env("RUSTC_WORKSPACE_WRAPPER", flowistry_rustc_path);
+    .env("RUSTC_WORKSPACE_WRAPPER", flowistry_rustc_path)
+    .args(&["rustc", "--profile", "check", "-vv"]);
 
-  match package {
-    Some(package) => {
-      cmd.arg("-p").arg(package);
+  // Add compile filter to specify the target corresponding to the given file
+  cmd.arg("-p").arg(&pkg.name);
+  let kind = &target.kind[0];
+  cmd.arg(format!("--{}", kind));
+  match kind.as_str() {
+    "lib" => {}
+    _ => {
+      cmd.arg(&target.name);
     }
-    None => panic!("Could not find package for file: {}", file_path.display()),
   };
 
+  // RNG is necessary to avoid caching
+  let n = thread_rng().gen::<u64>();
+  cmd.args(&["--", &format!("--flowistry={}", n)]).args(flags);
+
+  // FIXME(wcrichto): we should make these CLI args as well, then do
+  //   caching on VSCode's side
   for (k, v) in args {
     cmd.env(format!("FLOWISTRY_{}", k), v);
   }
